package djmil.cordacheckers.cordaclient;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import djmil.cordacheckers.cordaclient.dao.GameState;
import djmil.cordacheckers.cordaclient.dao.GameState.Status;
import djmil.cordacheckers.cordaclient.dao.Piece;
import djmil.cordacheckers.user.HoldingIdentityResolver;

@SpringBootTest
public class GameBoardTests {
    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    final String whitePlayerName = "alice";
    final String blackPlayerName = "bob";

    @Test
    void testSurrender() {
        final var hiWhite  = holdingIdentityResolver.getByUsername(whitePlayerName);
        final var hiBlack = holdingIdentityResolver.getByUsername(blackPlayerName);
        final String message = "GameBoard SURRENDER test";

        final GameState game = cordaClient.gameProposalCreate(
            hiWhite, hiBlack, Piece.Color.BLACK, message);

        System.out.println("Game UUID " +game.uuid());

        final GameState acceptedGameBlackView = cordaClient.gameProposalAccept(
            hiBlack, game.uuid());

        assertThat(acceptedGameBlackView.opponentColor()).isEqualByComparingTo(Piece.Color.WHITE);
        assertThat(acceptedGameBlackView.status()).isEqualByComparingTo(Status.GAME_BOARD_WAIT_FOR_OPPONENT);

        assertThatThrownBy(() -> { // Black can not surrender, since it is opponent's turn
            cordaClient.gameBoardSurrender(
                hiBlack, game.uuid());
        });

        final GameState surrendererGameView = cordaClient.gameBoardSurrender(
            hiWhite, game.uuid());

        assertThat(surrendererGameView.opponentName()).isEqualToIgnoringCase(blackPlayerName);
        assertThat(surrendererGameView.opponentColor()).isEqualByComparingTo(Piece.Color.BLACK);
        assertThat(surrendererGameView.status()).isEqualByComparingTo(Status.GAME_RESULT_YOU_LOOSE);

        final GameState winnerGameView = cordaClient.gameStateGet(
            hiBlack, game.uuid());

        assertThat(winnerGameView.opponentName()).isEqualToIgnoringCase(whitePlayerName);
        assertThat(winnerGameView.opponentColor()).isEqualByComparingTo(Piece.Color.WHITE);
        assertThat(winnerGameView.status()).isEqualByComparingTo(Status.GAME_RESULT_YOU_WON);
    }

}
